// Copyright (c) 2000,2001
// Naknan, Inc.
// All Rights Reserved
// $Id: JNISink.cpp,v 1.1 2001/07/05 18:16:44 james Exp $

#include "stdafx.h"
#include "JNISink.h"
#include <string.h>

JNISink::JNISink(JavaVM *vm, jobject callback)
{
	refCount = 0;
	jni_vm = vm;
	jni_callback = callback;
	lookupMethod = true;
	AddRef();
} // JNISink

JNISink::~JNISink()
{
	refCount = 0;
} // ~JNISink

HRESULT STDMETHODCALLTYPE JNISink::OnEvent( 
            /* [in] */ OPCHANDLE hClientSubscription,
            /* [in] */ BOOL bRefresh,
            /* [in] */ BOOL bLastRefresh,
            /* [in] */ DWORD dwCount,
            /* [size_is][in] */ ONEVENTSTRUCT __RPC_FAR *pEvents)
{
	JNIEnv *jni_env = NULL;

	jni_vm->AttachCurrentThread((void **)&jni_env, NULL);

	const char *UnKnown = "Unknown";

	// Only do this once...
	if (lookupMethod)
	{
		jclass clazz = jni_env->GetObjectClass(jni_callback);

		// Use "javap -s" to generate signature
		mid = jni_env->GetMethodID(clazz, "addEvent", 
			"(Ljava/lang/String;IIIIIIIILjava/lang/String;IIILjava/lang/String;Ljava/lang/String;IZIIIIIIIIILjava/lang/String;II)V");

		lookupMethod = false;
	}

	jstring source = jni_env->NewStringUTF(UnKnown);
	jstring message = jni_env->NewStringUTF(UnKnown);
	jstring condition = jni_env->NewStringUTF(UnKnown);
	jstring subcondition = jni_env->NewStringUTF(UnKnown);
	jstring actor = jni_env->NewStringUTF(UnKnown);

	int len = 0;

	if ((len = wcslen(pEvents->szSource)) > 0)
		source = jni_env->NewString(pEvents->szSource, len + 1);

	if ((len = wcslen(pEvents->szMessage)) > 0)
		message = jni_env->NewString(pEvents->szMessage, len + 1);

	if ((len = wcslen(pEvents->szConditionName)) > 0)
		condition = jni_env->NewString(pEvents->szConditionName, len + 1);

	if ((len = wcslen(pEvents->szSubconditionName)) > 0)
		subcondition = jni_env->NewString(pEvents->szSubconditionName, len + 1);

	if ((len = wcslen(pEvents->szActorID)) > 0)
		actor = jni_env->NewString(pEvents->szActorID, len + 1);

	SYSTEMTIME time;
    FileTimeToSystemTime(&pEvents->ftTime, &time);

	SYSTEMTIME activetime;
    FileTimeToSystemTime(&pEvents->ftActiveTime, &activetime);

	jni_env->CallVoidMethod(jni_callback, mid, source, time.wYear, time.wMonth, time.wDay,
		time.wDayOfWeek, time.wHour, time.wMinute, time.wSecond, time.wMilliseconds,
		message, pEvents->dwEventType, pEvents->dwEventCategory,
		pEvents->dwSeverity, condition, subcondition,
		pEvents->wQuality, pEvents->bAckRequired, activetime.wYear, activetime.wMonth, 
		activetime.wDay, activetime.wDayOfWeek, activetime.wHour, activetime.wMinute,
		activetime.wSecond, activetime.wMilliseconds, pEvents->dwCookie, actor,
		pEvents->wChangeMask, pEvents->wNewState);
	
	jni_vm->DetachCurrentThread();

	return S_OK;
} // OnEvent

HRESULT STDMETHODCALLTYPE JNISink::QueryInterface( 
            /* [in] */ REFIID riid,
            /* [iid_is][out] */ void __RPC_FAR *__RPC_FAR *ppvObject)
{
	if (ppvObject == NULL)
		return E_INVALIDARG;

	if (riid == IID_IUnknown)
	{
		*ppvObject = (IUnknown *)this;
	}
	else if (riid == IID_IOPCEventSink)
	{	
		*ppvObject = (IOPCEventSink *)this;
	}
	else
	{
		*ppvObject = NULL;
	}

	if (*ppvObject == NULL)
	{
		return E_NOINTERFACE;
	}

	AddRef();

	return S_OK;
} // QueryInterface

ULONG STDMETHODCALLTYPE JNISink::AddRef(void)
{
	return ++refCount;
} // AddRef

ULONG STDMETHODCALLTYPE JNISink::Release(void)
{
	ULONG currentCount = --refCount;

	if (currentCount == 0)
	{
		delete this;
	}

	return currentCount;
} // Release