// Copyright (c) 2000,2001
// Naknan, Inc.
// All Rights Reserved
// $Id: Opc.java,v 1.1 2001/07/05 18:16:44 james Exp $

package com.naknan.example.opc;

import java.util.*;
import java.text.*;

/**
 * <b><i>Opc</i></b><br>
 *
 * @author  Naknan
 * @version $Revision: 1.1 $
 */
public class Opc
	implements Runnable
{
    public native boolean connect_p(Opc ae_callback);
    public native boolean init_p(String host_name, String server_name);

	public static void main(String args[])
	{
		if (args.length < 2)
		{
			printHowToUse();
			return;
		}

		Opc opc_connection = new Opc(args[0], args[1]);
		(opc_connection.getThread()).start();
	} // main

	public static void printHowToUse()
	{
		System.err.println("java com.naknan.example.opc.Opc server host");
	} // printHowToUse

	public Opc(String _server_name, String _host_name)
	{
		server_name = _server_name;
		host_name = _host_name;
	} // Opc

	public Thread getThread()
	{
		if (thread == null)
			thread = new Thread(this);

		return thread;
	} // getThread

	private boolean init()
	{
		if (inited)
			return true;

		return (inited = init_p(host_name, server_name));
	} // init

	private boolean connect()
	{
		if (connected)
			return true;

		return (connected = connect_p(this));
	} // connect

	synchronized public void addEvent(String source, int ftyear, int ftmonth, int ftdayofmonth,
		int ftdayofweek, int fthour, int ftminute, int ftsecond, int ftmillisecond , String message,
		int type, int catagory, int severity, String condition, String subCondition, int quality,
		boolean ackreq, int ayear, int amonth, int adayofmonth, int adayofweek, int ahour, int aminute,
		int asecond, int amillisecond, int cookie, String actor, int changemask, int newstate)
	{
		/*
			'ftdayofweek' and 'adayofweek' are not compatible with 
			Java Calendar specification of the DAY_OF_WEEK
			so we will not use them. They are not needed to ensure the correct day any way.
		*/
		Calendar fttime = Calendar.getInstance();
		fttime.set(Calendar.YEAR, ftyear);
		fttime.set(Calendar.MONTH, ftmonth - 1); // Java MONTH counts the 12 months as 0 - 11
		fttime.set(Calendar.DAY_OF_MONTH, ftdayofmonth);
//		fttime.set(Calendar.DAY_OF_WEEK, ftdayofweek);
		fttime.set(Calendar.HOUR, fthour - 5); // Due to GMT/UTC (need to consider daylight savings time)
		fttime.set(Calendar.MINUTE, ftminute);
		fttime.set(Calendar.SECOND, ftsecond);
		fttime.set(Calendar.MILLISECOND, ftmillisecond);

		Calendar activetime = Calendar.getInstance();
		activetime.set(Calendar.YEAR, ayear);
		activetime.set(Calendar.MONTH, amonth - 1); // Java MONTH counts the 12 months as 0 - 11
		activetime.set(Calendar.DAY_OF_MONTH, adayofmonth);
//		activetime.set(Calendar.DAY_OF_WEEK, adayofweek);
		activetime.set(Calendar.HOUR, ahour - 5); // Due to GMT/UTC (need to consider daylight savings time)
		activetime.set(Calendar.MINUTE, aminute);
		activetime.set(Calendar.SECOND, asecond);
		activetime.set(Calendar.MILLISECOND, amillisecond);

		events.add(new OPCEVENT( source, fttime, message, type, catagory,
			severity, condition, subCondition, quality, ackreq, activetime, 
			cookie, actor, changemask, newstate));

		thread.interrupt();
	} // addEvent

    synchronized private void handleEvent()
    {
		Enumeration enum = events.elements();

		while (enum.hasMoreElements())
			System.out.println((OPCEVENT)enum.nextElement());

		events.clear();
    } // handleEvent

	public void run()
	{
		try
		{
			System.loadLibrary(libraryName);
		}
		catch(Exception e)
		{
			System.err.println("opc::run: Unable to load library - "+System.mapLibraryName(libraryName));
			return;
		}

		if (!inited)
			init();

		if (!connected)
			connect();

		while (connected)
		{
			try
			{
				if (events.size() > 0)
				{
					handleEvent();
				}
				Thread.sleep(1000 * 45); // Sleep 45 seconds or until woken up
			}
			catch (Exception e) {}
		}
	} // run

	private Thread thread = null;
	private boolean inited = false;
	private boolean connected = false;

	private String host_name = "";
	private String server_name = "";

	private Vector events = new Vector();
	private final static String libraryName = "OPC";

	class OPCEVENT
	{
		/*
			Client Severity		OPC Severity 

			HIGH				801 - 1000 
			MEDIUM HIGH			601 - 800 
			MEDIUM				401 - 600 
			MEDIUM LOW			201 - 400 
			LOW					1 - 200 
		*/

		public OPCEVENT(String _source, Calendar _fttime, String _message, int _type, int _catagory,
			int _severity, String _condition, String _subCondition, int _quality, boolean _ackreq,
			Calendar _activetime, int _cookie, String _actor, int _changemask, int _newstate)
		{
			source = _source.trim();
			fttime = new Long(_fttime.getTime().getTime());
			message = _message.trim();
			type = new Integer(_type).toString();
			catagory = new Integer(_catagory).toString();
			severity = new Integer(_severity).toString();
			condition = _condition.trim();
			subCondition = _subCondition.trim();
			quality = new Integer(_quality).toString();
			ackreq = _ackreq ? "Yes":"No";
			activetime = new Long(_activetime.getTime().getTime());
			cookie = new Integer(_cookie).toString();
			actor = _actor.trim();
			changemask = new Integer(_changemask).toString();
			newstate = new Integer(_newstate).toString();
		} // OPCEVENT

		public String toString()
		{
			StringBuffer sb = new StringBuffer();
			SimpleDateFormat sdf = new SimpleDateFormat ("MM.dd.yyy 'at' hh:mm:ss.ssss a"); 

			sb.append("Source " + source + "\n");
			sb.append("Time " + sdf.format(new java.util.Date(fttime.longValue())) + "\n");
			sb.append("Message " + message + "\n");
			sb.append("Type " + type + "\n");
			sb.append("Catagory " + catagory + "\n");
			sb.append("Severity " + severity + "\n");
			sb.append("ConditionName " + condition + "\n");
			sb.append("SubConditionName " + subCondition + "\n");
			sb.append("Quality " + quality + "\n");
			sb.append("AckRequired " + ackreq + "\n");
			sb.append("ActiveTime " + sdf.format(new java.util.Date(activetime.longValue())) + "\n");
			sb.append("Cookie " + cookie + "\n");
			sb.append("ActorID " + actor + "\n");
			sb.append("ChangeMask " + changemask + "\n");
			sb.append("NewState " + newstate + "\n");

			return sb.toString();
		} // toString

		String source;
		Long fttime;
		String message;
		String type;
		String catagory;
		String severity;
		String condition;
		String subCondition;
		String quality;
		String ackreq;
		Long activetime;
		String cookie;
		String actor;
		String changemask;
		String newstate;
	} // OPCEVENT
} // Opc
